#pragma once

#include "framework.h"

#define LIBRARY_VERSION "2.2.0"

#define REG_DMA_DATA_BASE           0x1000
#define REG_DMA_INDEX               0x1040
#define REG_DMA_DISABLE             0x1044
#define REG_INTERRUPT               0x100C
#define REG_INTERRUPT_MASK          0x1010
#define REG_CTR_TIMEOUT_ABSOLUTE    0x1050
#define REG_CTR_TIMEOUT_INTERVAL    0x1060
#define REG_DATA_COUNTER            0x1070
#define REG_SC_OUT                  0x1080
#define REG_SC_IN                   0x1084
#define REG_SC_INT_MASK             0x1088

#define REG_LBL_CONF_PCI_0(numCh)   (0x1400 + numCh * 0x40) 
#define REG_LBL_CONF_PCI_1(numCh)   (0x1404 + numCh * 0x40) 
#define REG_LBL_CONF_PCI_2(numCh)   (0x1408 + numCh * 0x40) 
#define REG_LBL_CONF_PCI_3(numCh)   (0x140C + numCh * 0x40) 
#define REG_LBL_CONF_PCI_4(numCh)   (0x1410 + numCh * 0x40) 
#define REG_LBL_CONF_PCI_5(numCh)   (0x1414 + numCh * 0x40) 
#define REG_LBL_CONF_PCI_6(numCh)   (0x1418 + numCh * 0x40) 
#define REG_LBL_CONF_PCI_7(numCh)   (0x141C + numCh * 0x40) 
#define REG_RX_CONF(numCh)          (0x1420 + numCh * 0x40) 

#define REG_TX_CONF(numCh)          (0x1428 + numCh * 0x40) 
#define REG_TX_CONTROL(numCh)       (0x142C + numCh * 0x40) 
#define REG_TX_FIFO(numCh)          (0x1430 + numCh * 0x40) 
#define REG_TX_DATA_RAM(numCh)      (0x8000 + numCh * 0x800)
#define REG_TX_DESC_RAM(numCh)      (0x8400 + numCh * 0x800)

#define A429_MODE_0         0               ///<   0
#define A429_MODE_1         1               ///<   1
#define A429_MODE_2         2               ///<   2
#define A429_MODE_3         3               ///<   3

#define A429_STOP           0               ///< 
#define A429_START          ~A429_STOP      ///< 

#define A429_OFF            0               ///< 
#define A429_ON             ~A429_OFF       ///< 

#define A429_FALSE          0               ///< 
#define A429_TRUE           ~A429_FALSE     ///< 

#define A429_SPEED_100          0       ///< 100 /
#define A429_SPEED_12_14        1       ///< 12.5 /
#define A429_SPEED_50           2       ///< 50 /
#define A429_SPEED_12           3       ///< 12 /
#define A429_SPEED_14_5         4       ///< 14.5 /

#define A429_TXRR_10MS      0       ///< txrr 10 ms
#define A429_TXRR_1MS       1       ///< txrr 1 ms

#define A429_SC_OUT_1           (1)         ///<    0
#define A429_SC_OUT_0           (1<<1)      ///<    1
#define A429_SC_OUT_NO_ACT      0           ///<    

#define A429_IN_RK_1    0       ///<   1
#define A429_IN_RK_2    1       ///<   2
#define A429_IN_RK_3    2       ///<   3
#define A429_IN_RK_4    3       ///<   4
#define A429_IN_RK_5    4       ///<   4
#define A429_IN_RK_6    5       ///<   4
#define A429_IN_RK_7    6       ///<   4
#define A429_IN_RK_8    7       ///<   4

#define A429_SC_RISE    0                   ///<  
#define A429_SC_FALL    ~A429_SC_RISE       ///<  

/// \brief channel info
typedef struct sA429_CHANNEL_INFO {
	WCHAR* device_name;		///   
	UINT32 g_channel;		///   
	UINT32 i_channel;		///    
	UINT32 b_channel;		///    
	UINT32 g_board;			///  
	UINT32 channel_type;		///   (T_CHANNEL_RX  T_CHANNEL_TX)
	HANDLE arinc;			///  
} A429_CHANNEL_INFO, A429_DEVICE_INFO;

typedef struct A429_DEVICE_LIST {
	UINT32 count;				///  
	UINT32 pcount;				///  

	A429_CHANNEL_INFO* device;	///  
	A429_CHANNEL_INFO* pdev;	///  
} A429_DEVICE_LIST, *PA429_DEVICE_LIST;


/**
* @brief   
* 
*          ID GUID_DEVINTERFACE_ARINC429, 
*  ,         -   .  
* A429_DEVICE_LIST      a429_free.
*
*
* @return    A429_DEVICE_LIST
*/
A429_DEVICE_LIST* a429_enumerate();

/**
* @brief  
*/
VOID a429_d();

/**
* @brief   
* 
* @param deviecs    A429_DEVICE_LIST
*/
VOID a429_free(A429_DEVICE_LIST* devices);

/**
* @brief   
* 
*             A429_CHANNEL_INFO.
*             CloseHandle.
* 
* @param    A429_CHANNEL_INFO
* 
* @return    NULL,   
*/
HANDLE a429_open(A429_CHANNEL_INFO* info);

/**
* @brief   
*
* @param arinc    
* @param addr 
* @param pbuf   UINT32,     
* @return    0   
*/
UINT32 a429_read_reg(A429_CHANNEL_INFO* arinc, UINT32 addr, UINT32* pbuf);

/**
* @brief   
*
* @param arinc    
* @param addr 
* @param buf  
* @return    0   
*/
UINT32 a429_write_reg(A429_CHANNEL_INFO* arinc, UINT32 addr, UINT32 buf);

/**
* @brief   
*
* @param arinc    
* @param addr 
* @param buf  
* @param mask 
* @return    0   
*/
UINT32 a429_modify_reg(A429_CHANNEL_INFO* arinc, UINT32 addr, UINT32 buf, UINT32 mask);

/**
* @brief  DMA
* 
* @param arinc    
* @param channel   
* @param pcount      ,         
* @param dmabuf   ,     DMA
* 
* @return    0   
*/
UINT32 a429_read_dma(A429_CHANNEL_INFO* arinc, UINT32* pcount, DMA_BLOCK** dmabuf);

/**
* @brief  DMA
* 
*   DMA    
* 
* @param arinc    
* 
* @return    0   
*/
UINT32 a429_clear_dma(A429_CHANNEL_INFO* arinc);

/**
* @brief    
* 
* @param arinc    
* @param version         
* 
* @return    0   
*/
UINT32 a429_get_device_info(A429_CHANNEL_INFO* arinc, VERSION_REQ* version);

/**
* @brief  DMA 
* 
* @param arinc    
* @param value A429_OFF  A429_ON
* 
* @return    0   
*/
UINT32 a429_manageDma(A429_CHANNEL_INFO* arinc, UINT32 value);

/**
* @brief  DMA 
*
* @param arinc    
* @param value A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a429_manageChannelDma(A429_CHANNEL_INFO* arinc, UINT32 value);

/**
* @brief  RAM 
*
* @param arinc    
* @param container    RAM_REQ  ,     
*
* @return    0   
*/
UINT32 a429_writeRam(A429_CHANNEL_INFO* arinc, RAM_REQ* container);

/**
* @brief  RAM 
*
* @param arinc    
* @param container    RAM_REQ,     
*
* @return    0   
*/
UINT32 a429_readRam(A429_CHANNEL_INFO* arinc, RAM_REQ* container);

/**
* @brief     
*
* @param arinc    
* @param info    T_INT_REQ
*
* @return    0   
*/
UINT32 a429_manageTrackingInterrupt(A429_CHANNEL_INFO* arinc, T_INT_REQ* info);

/**
* @brief     
*
* @param arinc    
* @param info    T_INT_REQ,     
*
* @return    0   
*/
UINT32 a429_checkTrackingInterrupt(A429_CHANNEL_INFO* arinc, T_INT_REQ* info);

/**
* @brief    
*
* @param arinc    
* @param action A429_STOP  A429_START
*
* @return    0   
*/
UINT32 a429_txStartStop(A429_CHANNEL_INFO* arinc, UINT32 action);

/**
* @brief   
*
* @param arinc    
*
* @return    0   
*/
UINT32 a429_txStartOnce(A429_CHANNEL_INFO* arinc);

/**
* @brief    
*
* @param arinc    
* @param mode  
*
* @return    0   
*/
UINT32 a429_txSetMode(A429_CHANNEL_INFO* arinc, UINT32 mode);

/**
* @brief     FIFO 
*
* @param arinc    
* @param data  
*
* @return    0   
*/
UINT32 a429_writeTxFifo(A429_CHANNEL_INFO* arinc, UINT32 data);

/**
* @brief   
*
* @param arinc    
* @param numReg   REG_LBL_CONF_PCI_0 - REG_LBL_CONF_PCI_7
* @param mask 
* @param value 
*
* @return    0   
*/
UINT32 a429_setLabelConfReg(A429_CHANNEL_INFO* arinc, UINT32 numReg, UINT32 mask, UINT32 value);

/**
* @brief    
*
* @param arinc    
* @param speed  (A429_SPEED_...)
*
* @return    0   
*/
UINT32 a429_setSpeed(A429_CHANNEL_INFO* arinc, UINT32 speed);

/**
* @brief   
*
* @param arinc    
* @param action A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a429_manageEnBit(A429_CHANNEL_INFO* arinc, UINT32 action);

/**
* @brief    
*
* @param arinc    
* @param state   ,         - A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a429_isEnBit(A429_CHANNEL_INFO* arinc, UINT32* state);

/**
* @brief   
*
* @param arinc    
* @param parcheck  30  RX_CONF_REG  TX_CONF_REG
* @param parity  29  RX_CONF_REG  TX_CONF_REG
*
* @return    0   
*/
UINT32 a429_parityManage(A429_CHANNEL_INFO* arinc, UINT32 parcheck, UINT32 parity);

/**
* @brief    
*
* @param arinc    
* @param action A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a429_rxFiltration(A429_CHANNEL_INFO* arinc, UINT32 action);

/**
* @brief    
*
* @param arinc    
* @param label 
* @param action A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a429_rxManageFiltrationLabel(A429_CHANNEL_INFO* arinc, UINT32 label, UINT32 action);

/**
* @brief    
*
* @param arinc    
*
* @return    0   
*/
UINT32 a429_rxClearFiltrationLabel(A429_CHANNEL_INFO* arinc);

/**
* @brief   9  10 
*
* @param arinc    
* @param action      RCV_SDI 
* @param bit9   9
* @param bit10   10
*
* @return    0   
*/
UINT32 a429_rxManageRcvSdi(A429_CHANNEL_INFO* arinc, UINT32 action, UINT32 bit9, UINT32 bit10);

/**
* @brief  
*
* @param arinc    
*
* @return    0   
*/
UINT32 a429_deinit(A429_CHANNEL_INFO* arinc);

/**
* @brief       
*
* @param arinc    
* @param action A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a429_manageReverce(A429_CHANNEL_INFO* arinc, UINT32 action);

/**
* @brief   
*
* @param arinc    
* @param gapBit  ,  19-25  REG_TX_CONF
*
* @return    0   
*/
UINT32 a429_txGapBits(A429_CHANNEL_INFO* arinc, UINT32 gapBit);

/**
* @brief    RRT
*
* @param arinc    
* @param txrr A429_TXRR_10MS  A429_TXRR_1MS 
*
* @return    0   
*/
UINT32 a429_txRrD(A429_CHANNEL_INFO* arinc, UINT32 txrr);

/**
* @brief   RRT
*
* @param arinc    
* @param action A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a429_txSkipRrt(A429_CHANNEL_INFO* arinc, UINT32 action);

/**
* @brief    
*
* @param arinc    
*
* @return    0   
*/
UINT32 a429_clearScIntMask(A429_CHANNEL_INFO* arinc);

/**
* @brief   
*
* @param arinc    
* @param action1    1
* @param action2    2
* @param action3    3
* @param action4    4
*
* @return    0   
*/
UINT32 a429_manageScOut(A429_CHANNEL_INFO* arinc, UINT32 action1, UINT32 action2, UINT32 action3, UINT32 action4, UINT32 action5, UINT32 action6, UINT32 action7, UINT32 action8);

/**
* @brief    
*
* @param arinc    
* @param numRk	  
* @param stateRk A429_SC_RISE  A429_SC_FALL
* @param actionRk A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a429_manageScIntMask(A429_CHANNEL_INFO* arinc, UINT32 numRk, UINT32 stateRk, UINT32 actionRk);

/**
* @brief    RRT
*
* @param arinc    
* @param rrtValue   RRT
*
* @return    0   
*/
UINT32 a429_txStartWithRrt(A429_CHANNEL_INFO* arinc, UINT32 rrtValue);

/**
* @brief     RRT
*
* @param arinc    
* @param rrtValue   RRT
*
* @return    0   
*/
UINT32 a429_txStartWithRrtOnce(A429_CHANNEL_INFO* arinc, UINT32 rrtValue);

/**
* @brief      
*
* @param arinc    
* @param rknum   
*
* @return    0   
*/
UINT32 a429_txSetRequestTransferByRkIn(A429_CHANNEL_INFO* arinc, UINT32 rknum);

/**
* @brief   
*
* @param arinc    
* @param mask 
* @param value 
*
* @return    0   
*/
UINT32 a429_manageInterruptMask(A429_CHANNEL_INFO* arinc, UINT32 mask, UINT32 value);

/**
* @brief     
*
* @param arinc    
* @param rknum   
*
* @return    0   
*/
UINT32 a429_rxSetReceiveReadyByRkIn(A429_CHANNEL_INFO* arinc, UINT32 rknum);

/**
* @brief  RAM 
*
* @param arinc    
*
* @return    0   
*/
UINT32 a429_ramCls(A429_CHANNEL_INFO* arinc);