/**
 * @file ioctl.h
 * @brief   -  ioctl   ,     
 */

#pragma once

#ifndef __IOCTL_H
#define __IOCTL_H

/** GUID   */
DEFINE_GUID(GUID_DEVINTERFACE_ARINC429, 0x3BD2B180, 0xD211, 0x4D88, 0x8F, 0x46, 0xB7, 0x38, 0x45, 0xCF, 0x04, 0x29); 

#define VENDOR_ID_NOVOMAR		0xA203
#define	DEVICE_ID_PCIE_A429		0x9477
#define	DEVICE_ID_MPCIE_A429	0x9478
#define	DEVICE_ID_XMC_A429		0x9481

#define	DEVICE_REV_ID_A429_UD88	0x01
#define	DEVICE_REV_ID_A429_UD84	0x11
#define	DEVICE_REV_ID_A429_UD44	0x21
#define	DEVICE_REV_ID_A429_UD66	0x31
#define	DEVICE_REV_ID_A429_UD22	0x41

/**   IOCTL */

/**    */
#define IOCTL_READ_BAR					CTL_CODE(FILE_DEVICE_CONTROLLER, 0x100, METHOD_BUFFERED, FILE_ANY_ACCESS) 
/**    */
#define IOCTL_WRITE_BAR					CTL_CODE(FILE_DEVICE_CONTROLLER, 0x101, METHOD_BUFFERED, FILE_ANY_ACCESS) 
/**      */
#define IOCTL_WRITE_BAR_MASK			CTL_CODE(FILE_DEVICE_CONTROLLER, 0x102, METHOD_BUFFERED, FILE_ANY_ACCESS) 
/**    */
#define IOCTL_READ_RAM					CTL_CODE(FILE_DEVICE_CONTROLLER, 0x103, METHOD_BUFFERED, FILE_ANY_ACCESS) 
/**    */
#define IOCTL_WRITE_RAM					CTL_CODE(FILE_DEVICE_CONTROLLER, 0x104, METHOD_BUFFERED, FILE_ANY_ACCESS) 

/**  DMA */
#define IOCTL_CLEAN_DMA					CTL_CODE(FILE_DEVICE_CONTROLLER, 0x200, METHOD_BUFFERED, FILE_ANY_ACCESS)
/**  DMA */
#define IOCTL_READ_DMA					CTL_CODE(FILE_DEVICE_CONTROLLER, 0x201, METHOD_BUFFERED, FILE_ANY_ACCESS)

/**      */
#define IOCTL_VERSION					CTL_CODE(FILE_DEVICE_CONTROLLER, 0x300, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_INFO_CHANNEL				CTL_CODE(FILE_DEVICE_CONTROLLER, 0x302, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define	IOCTL_INFO_DEVICE				CTL_CODE(FILE_DEVICE_CONTROLLER, 0x303, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_TRACKING_INT				CTL_CODE(FILE_DEVICE_CONTROLLER, 0x304, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_CHECK_TRACKING_INT		CTL_CODE(FILE_DEVICE_CONTROLLER, 0x305, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_START_AS_RK_IN			CTL_CODE(FILE_DEVICE_CONTROLLER, 0x306, METHOD_BUFFERED, FILE_ANY_ACCESS)


/**   */



/// \brief / 
typedef struct _BAR_REG_REQ{
	UINT32	regAddr;	///  
	UINT32	data;		/// 
} BAR_REG_REQ, *PBAR_REG_REQ;

/// \brief    
typedef struct _BAR_REG_REQ_MASK{
	UINT32	regAddr;	///  
	UINT32	data;		/// 
	UINT32	mask;		/// 
} BAR_REG_REQ_MASK, *PBAR_REG_REQ_MASK;


/// \brief  
#define RAM_REQ_MAX 256

#define RAM_BLOCK_TYPE_DATA             1
#define RAM_BLOCK_TYPE_DESCRIPTOR       2

/// \brief /  RAM 
typedef struct _RAM_REQ{
	UINT8 channel;					///   
	UINT32 type;					///   (DATA - 1, DESCRIPTOR - 2)
	UINT32 offset;					///    
	UINT32 length;					///   
	UINT32 dwords[RAM_REQ_MAX];		///  
} RAM_REQ, *PRAM_REQ;


/// \brief   DMA  
#define DMA_COUNT_BLOCKS 256

/// \brief  DMA
typedef struct _DMA_BLOCK{
	UINT32 w1;						///  1
	UINT32 w2;						///  2
	UINT32 w3;						///  3
	UINT32 w4;						///  4
} DMA_BLOCK, *PDMA_BLOCK;

/// \brief ioctl    DMA
typedef struct _DMA_CONTAINER{
	UINT8 channel;						///   
	UINT32 count;						///  
	DMA_BLOCK blocks[DMA_COUNT_BLOCKS];	///  
} DMA_CONTAINER, *PDMA_CONTAINER;

/// \brief ioctl   DMA
typedef struct _DMA_REQ{
	UINT8 channel;						///   
	UINT32 count;						///	  
} DMA_REQ, *PDMA_REQ;


#define T_CHANNEL_RX	0				///   - 
#define	T_CHANNEL_TX	~T_CHANNEL_RX	///   - 

#define T_BLOCK_DMA_TYPE_CH(word1)  ((word1 >> 31) & 0x1)
#define T_BLOCK_DMA_NUM_CH(word1)   ((word1 >> 24) & 0x7)


#define T_TRACKING_INT_MAN_OFF      0
#define T_TRACKING_INT_MAN_ON       ~T_TRACKING_INT_MAN_OFF

#define T_CHECK_TRACKING_INT_CNT 8

#define T_CHECK_TRACKING_INT_RKIN_1 0
#define T_CHECK_TRACKING_INT_RKIN_2 1
#define T_CHECK_TRACKING_INT_RKIN_3 2
#define T_CHECK_TRACKING_INT_RKIN_4 3
#define T_CHECK_TRACKING_INT_RKIN_5 4
#define T_CHECK_TRACKING_INT_RKIN_6 5
#define T_CHECK_TRACKING_INT_RKIN_7 6
#define T_CHECK_TRACKING_INT_RKIN_8 7

/// \brief    
typedef struct _T_INT_REQ{
	UINT8 channel;										///   
	UINT32 manage;										/// / 
	UINT64 startFreeTimer;								///   freetimer
	UINT64 finishFreeTimer;								///   freetimer
	UINT32 rkInAppearCount[T_CHECK_TRACKING_INT_CNT];	///   
} T_INT_REQ, *PT_INT_REQ;

#define RKIN_NONE      0
#define RKIN_NUM_1     1
#define RKIN_NUM_2     2
#define RKIN_NUM_3     3
#define RKIN_NUM_4     4
#define RKIN_NUM_5     5
#define RKIN_NUM_6     6
#define RKIN_NUM_7     7
#define RKIN_NUM_8     8

#define FRONT_DOWN      0
#define FRONT_UP        ~FRONT_DOWN


/// \brief    
typedef struct _VERSION_REQ{
	UINT32 device_id;			/// id 
	UINT32 vendor_id;			/// id 
	UINT32 revision_id;			///  

	UINT64 mbar_addr;			///  BAR ()
	UINT64 pbar_addr;			///  BAR ()
	UINT32 irq;					///  
	UINT64 dma_size;			///   DMA
	LARGE_INTEGER dma_addr_device;		///  DMA  
	UINT64 dma_addr_driver;				///  DMA  

	UINT32 driver_version;		///  
	UINT32 driver_build_date;	///   

	UINT8 rx_channel_count;	///  -
	UINT8 tx_channel_count;	///  -
	UINT8 sc_tx_count;		///   
	UINT8 sc_rx_count;		///   
	UINT32 g_channel_offset; ///     

} VERSION_REQ, *PVERSION_REQ;

#endif